#!/bin/bash

## VARS
yes="$(gettext "y")"
no="$(gettext "n")"
ok="$(gettext "  OK  ")"
fail="$(gettext "FAILED")"
passed="$(gettext "PASSED")"
warning="$(gettext "WARNING")"
critical="$(gettext "CRITICAL")"

## COLOR FUNCTIONS

RES_COL="60"
MOVE_TO_COL="\\033[${RES_COL}G"
SETCOLOR_INFO="\\033[1;38m"
SETCOLOR_SUCCESS="\\033[1;32m"
SETCOLOR_FAILURE="\\033[1;31m"
SETCOLOR_WARNING="\\033[1;33m"
SETCOLOR_NORMAL="\\033[0;39m"

#----
## print info message
## @param       message info
## @param       type info (ex: INFO, username...)
## @Stdout      info message
#----
function echo_info() {
    echo -e "${1}${MOVE_TO_COL}[${SETCOLOR_INFO}${2}${SETCOLOR_NORMAL}]"
}

#----
## print success message
## @param       message
## @param       word to specify success (ex: OK)
## @Stdout      success message
#----
function echo_success() {
    echo -e "${1}${MOVE_TO_COL}[${SETCOLOR_SUCCESS}${2}${SETCOLOR_NORMAL}]"
}

#----
## print failure message
## @param       message
## @param       word to specify failure (ex: fail)
## @Stdout      failure message
#----
function echo_failure() {
    echo -e "${1}${MOVE_TO_COL}[${SETCOLOR_FAILURE}${2}${SETCOLOR_NORMAL}]"
}

#----
## print passed message
## @param       message
## @param       word to specify pass (ex: passed)
## @Stdout      passed message
#----
function echo_passed() {
    echo -e "${1}${MOVE_TO_COL}[${SETCOLOR_WARNING}${2}${SETCOLOR_NORMAL}]"
}

#----
## print warning message
## @param       message
## @param       word to specify warning (ex: warn)
## @Stdout      warning message
#----
function echo_warning() {
    echo -e "${1}${MOVE_TO_COL}[${SETCOLOR_WARNING}${2}${SETCOLOR_NORMAL}]"
}

#----
## make a question with yes/no possiblity
## use "no" response by default
## @param       message to print
## @param       default response (default to no)
## @return 0    yes
## @return 1    no
#----
function yes_no_default() {
        local message=$1
        local default=${2:-$no}
        local res="not_define"
        while [ "$res" != "$yes" ] && [ "$res" != "$no" ] && [ ! -z "$res" ] ; do
                echo -e "\n$message\n$(gettext "[y/n], default to [$default]:")"
                echo -en "> "
                read res
                [ -z "$res" ] && res="$default"
        done
        if [ "$res" = "$yes" ] ; then
                return 0
        else
                return 1
        fi
}



#----
## print a message, test directory if answer is correct
## Loop if not a valid directory
## @param       message
## @param       default value (use "NO_DEFAULT" if not default value)
## @param       global variable to set a result
## @return 0    end
#----
function answer_with_testdir() {
        local message=$1
        local default=$2
        local var_ref=$3
        local res="not_define"
        local first=0
        while [ ! -d "$res" ] ; do
                [ $first -eq 1 ] && echo_passed "$(gettext "$res is not a directory or does not exist.")" "$warning"
                echo -e "\n$message"
                [ "$default" != "NO_DEFAULT" ] && echo -e "$(gettext "default to [$default]")"
                echo -en "> "
                read res
                if [ -z "$res" ] ; then
                        [ $default != "NO_DEFAULT" ] && res=$default
                fi
                if [ -z ${res#/} ] ; then
                        echo_passed  "$(gettext "/ is not a suitable path...")" "$warning"
                        res="not_define"
                else
                        first=1
                fi
        done
        eval $var_ref=$res
        return 0
}

#----
## print a message, create directory with answer
## Loop if not a valid directory (slash...)
## @param       message
## @param       default value (use "NO_DEFAULT" if not default value)
## @param       global variable to set a result
## @return 0    end
#----
function answer_with_createdir() {
        local message=$1
        local default=$2
        local var_ref=$3
        local res="not_define"
        local first=0
        while [ ! -d "$res" ] ; do
                [ $first -eq 1 ] && echo_passed "$(gettext "Directory $res does not exists.")" "$fail"
                echo -e "\n$message"
                [ "$default" != "NO_DEFAULT" ] && echo -e "$(gettext "default to") [$default]"
                echo -en "> "
                read res
                if [ -z "$res" ] ; then
                        [ "$default" != "NO_DEFAULT" ] && res=$default
                fi
                if [ -z "${res#/}" -o "$yes" = "$res" -o "$no" = "$res" ] ; then
                        echo_passed  "$(gettext "You select slash...")"
                        res="not_define"
                else
                        first=1
                        [ -d "$res" ] && break
                        yes_no_default "$(gettext "Do you want me to create this directory ?") [$res]" "y"
                        if [ $? -eq 0 ] ; then
                                mkdir -p $res
                                if [ $? -ne 0 ] ; then
                                        echo_passed "$(gettext "Could not create directory.")" "$fail"
                                        #continue
                                fi
                        fi
                fi
        done
        eval $var_ref=$res
        return 0
}

#----
## print a message, test if file exists
## Loop if not a valid file
## @param       message
## @param       default value (use "NO_DEFAULT" if not default value)
## @param       global variable to set a result
## @return 0    end
#----
function answer_with_testfile() {
        local message=$1
        local default=$2
        local var_ref=$3
        local res="not_define"
        local first=0
        while [ ! -f "$res" ] ; do
                [ $first -eq 1 ] && echo_passed "$(gettext "$res is not a valid file.")" "$critical"
                echo -e "\n$message"
                [ "$default" != "NO_DEFAULT" ] && echo -e "$(gettext "default to") [$default]"
                echo -en "> "
                read res
                if [ -z "$res" ] ; then
                        [ "$default" != "NO_DEFAULT" ] && res=$default
                fi
                first=1
        done
        eval $var_ref=$res
        return 0
}


#----
## Test if file exist
## @param       file
## @param       variable to reset
## @return 0    file exist
## @return 1    file does not exist
#----
function testfile_clean() {
        local file="$1"
        local var="$2"
        [ -z "$file" ] && return 1
        if [ ! -e "$file" ] ; then
                if [ ! $var = "" ] ; then
			eval $var=""
		fi
                return 1
        else
                return 0
        fi
}


#----
## Test if directory exist
## @param       directory
## @param       variable to reset
## @return 0 directory exist
## @return 1 directory does not exist
#----
function testdir_clean() {
        local dir="$1"
        local var="$2"
        [ -z "$dir" ] && return 1
        if [ ! -d "$dir" ] ; then
                eval $var=""
                return 1
        else
                return 0
        fi
}

function test_answer() {
        if [ ! -z $2 ] ; then
                if [ $2 != "" ] ; then
                        eval $1=$2
                fi
        fi
}

#----
## Check result and print a message
## @param       return code to check
## @param       message to print
#----
function check_result() {
        local code=$1
        shift
        local message=$@

        if [ $code -eq 0 ] ; then
                echo_success "$message" "$ok"
        else
                echo_failure "$message" "$fail"
        fi
        return 0
}

BASE_DIR=$(dirname $0)
BASE_DIR=$( cd $BASE_DIR; pwd )
export BASE_DIR

echo "BMC Service Desk Express Nagios Connector Installer"
echo "Copyright (c) 2010 GroupeOpen - http://www.open-groupe.com/"
echo "Developped by : Herve Roux under GPL Licence 2.0."

eventhandlers_dir=`locate -b '\eventhandlers' 2>/dev/null | head -1 2>/dev/null`
answer_with_testdir 'Where do you want to install SDE Nagios Connector eventhandler script?' "$eventhandlers_dir" eventhandlers_dir

# Copy Script
#########################
echo -n "Copying eventhandler script"
cp -f $BASE_DIR/sde.pl $eventhandlers_dir 2>/tmp/sde2nagios_error
check_result $?
cat /tmp/sde2nagios_error

conf_file_dir="/etc/nagios/sde"
answer_with_createdir 'Where do you want to install SDE Nagios Connector configuration files?' "$conf_file_dir" conf_file_dir

# Copy conf file
#########################

> /tmp/sde2nagios_error
print_warn=0
com_file_name=$conf_file_dir/sde_commands.cfg

testfile_clean $com_file_name
if [ $? -eq 0 ] ; then
	mv -f $com_file_name $com_file_name.old 2>>/tmp/sde2nagios_error
	echo "'sde_commands.cfg' already exist in '$conf_file_dir', old command file was renamed as 'sde_commands.cfg.old'" >> /tmp/sde2nagios_error
	print_warn=1
fi

testfile_clean $conf_file_dir/sde.conf
if [ $? -eq 0 ] ; then
        conf_file_name=$conf_file_dir/sde.conf.new
	echo "'sde.conf' already exist in '$conf_file_dir', copying as 'sde.conf.new'" >> /tmp/sde2nagios_error
        print_warn=1
else
        conf_file_name=$conf_file_dir/sde.conf
fi

echo -n "Copying configuration files"
#cp -f $BASE_DIR/sde_commands.cfg $com_file_name 2>>/tmp/sde2nagios_error
conf_file_name_for_sed=$conf_file_dir/sde.conf
conf_file_name_for_sed=`echo "${conf_file_name_for_sed/:/\\:}"`
sed "s:<CUSTOM_FILE>:$conf_file_name_for_sed:g" $BASE_DIR/sde_commands.cfg > $com_file_name 2>>/tmp/sde2nagios_error
myres=$?
cp -f $BASE_DIR/sde.conf $conf_file_name 2>>/tmp/sde2nagios_error
let myres=$myres+$?
if [ $myres -eq 0 ] ; then
	if [ $print_warn -eq 0 ] ; then
		echo_success "" "$ok"
	else
		echo_passed "" "$passed"
	fi
	cat /tmp/sde2nagios_error
	if [ ! $conf_file_dir = "/etc/nagios/sde" ] ; then
		echo -e "${SETCOLOR_FAILURE}WARNING : You have chosen to use a different path than the default one for the configuration files. Don't forget to use the '-c $conf_file_name' option when calling 'sde.pl'. Alternatively you can create a symbolic link to '/etc/nagios/sde/sde.conf'.${SETCOLOR_NORMAL}"
	fi

else
	echo_failure "" "$fail"
	cat /tmp/sde2nagios_error
fi


# Setting up nagios owner
chown nagios:nagios $conf_file_dir 1>/dev/null 2>&1
chown nagios:nagios $conf_file_name 1>/dev/null 2>&1
chown nagios:nagios $com_file_name 1>/dev/null 2>&1

yes_no_default 'Do you want to add SDE global eventhandler to your nagios configuration file?' 'y'
if [ $? -eq 0 ] ; then
	nagios_conf_file=`locate -b '\nagios.cfg' 2>/dev/null | head -1 2>/dev/null`
	answer_with_testfile 'Where is your nagios configuration file?' "$nagios_conf_file" nagios_conf_file
	echo -n "Updating Nagios Configuration file"
	myres=0
	exist_in_conf=`grep -x "cfg_file=$com_file_name" $nagios_conf_file | wc -l`
	if [ $exist_in_conf -eq 0 ] ; then
		echo "cfg_file=$com_file_name" >> $nagios_conf_file
		let myres=$myres+$?
	fi
	exist_in_conf=`grep -x "global_host_event_handler=log-host-event-to-sde" $nagios_conf_file | wc -l`
	if [ $exist_in_conf -eq 0 ] ; then
		echo "global_host_event_handler=log-host-event-to-sde" >> $nagios_conf_file
        	let myres=$myres+$?
	fi
	exist_in_conf=`grep -x "global_service_event_handler=log-service-event-to-sde" $nagios_conf_file | wc -l`
	if [ $exist_in_conf -eq 0 ] ; then
		echo "global_service_event_handler=log-service-event-to-sde" >> $nagios_conf_file
        	let myres=$myres+$?
	fi
	check_result $myres
	echo	
	service nagios reload
fi

yes_no_default 'Do you want to run the check list?' 'y'
if [ $? -eq 0 ] ; then
	if [ $conf_file_dir = "/etc/nagios/sde" ] ; then
		Check_OPT=""
	else
		Check_OPT=" -c $conf_file_name"
	fi
	echo
	echo "Running '$eventhandlers_dir/sde.pl --check$Check_OPT'"
	$eventhandlers_dir/sde.pl --check$Check_OPT
	if [ ! $? -eq 0 ] ; then
		echo "Installation done with some Warning"
		echo "Try running again '$eventhandlers_dir/sde.pl --check$Check_OPT' after fixing the issues."	
	fi
fi



